(ns clojsa.handler
  (:require [clojsa.views :as views]
            [clojsa.saclient :as client]
            [clojsa.saparser :as parser]
            [compojure.core :refer [defroutes GET POST]]
            [compojure.route :as route]
            [compojure.coercions :refer [as-int]]
            [ring.middleware.defaults :refer [wrap-defaults site-defaults]]
            [ring.middleware.session :refer [wrap-session]]
            [ring.middleware.session.cookie :refer (cookie-store)]
            [ring.adapter.jetty :as jetty]
            [environ.core :refer [env]]))


(defn get-thread [session id page]
  (let [turl (client/thread-url id page)
        tresp (client/text-response session turl)]
    (parser/thread-map id page tresp)))

(defn thread-page [session id page]
  (let [thread (get-thread session id page)
        {:keys [id page page-count title]} thread
        login-part (views/login-form
                    ["/thread/%d?page=%d" id page]
                    (get session :loggedin false))
        header-part (views/header-fragment login-part)
        thread-part (views/thread-page thread)
        paginate-part (views/paginate
                       (str "/thread/" id) page page-count)]
    (views/main-template {:title title}
                         header-part thread-part paginate-part)))

(defn get-bookmarks [session page]
  (let [burl (client/bookmarks-url page)
        bresp (client/text-response session burl)]
    (parser/bookmarks-map page bresp)))

(defroutes app-routes
  (GET "/" request (views/index-page request))

  (GET "/login" []
       (views/main-template
        {:title "Login"}
        (-> (views/login-form ["/bookmarks"] false)
                               (views/login-page))))

  (POST "/login" [username password next :as {session :session}]
        (let [resp-cookies (client/login-response username password session)]
          {:status 302
           :headers {"Location" next}
           :session (assoc session :cookies resp-cookies :loggedin true)}))

  (GET "/bookmarks" [page :<< as-int
                     :as {session :session}]
       (if (get session :loggedin)
           (let [beems (get-bookmarks session page)]
             (views/bookmarks-page beems))
           {:status 302
            :headers {"Location" "/login"}}))

  (GET "/bookmarks" []
       {:status 302 :headers {"Location" "/bookmarks?page=1"}})

  (GET "/thread/:id" [id :<< as-int
                      page :<< as-int
                      :as {session :session}]
       (thread-page session id page))

  (GET "/thread/:id" [id]
       {:status 302 :headers {"Location" (str "/thread/" id "?page=1")}})

  (route/not-found "Not Found"))

(def app
  (-> app-routes
      (wrap-defaults site-defaults)
      (wrap-session {:cookie-attrs {:max-age (* 7 24 3600)}
                     :store (cookie-store {:key (or (env :cskey)
                                                    "699a7e4bd79b3f56")})})))

(defn -main [& [port]]
  (let [port (Integer. (or port (env :port) 5000))]
    (jetty/run-jetty app {:port port :join? false})))
