(ns clojsa.saparser
  (:require [clojure.string :as string]
            [clojure.pprint :refer [pprint]]
            [hickory.core :refer :all]
            [hickory.select :as s]
            [hickory.convert :refer [hickory-to-hiccup]]))

(defn hickory-doc [doc]
  (-> doc parse as-hickory))

(defn hickory-div [content class]
  {:type :element
   :attrs {:class class}
   :tag :div
   :content content})

(defn element-node [kw htree]
  (case kw

    :author
    (-> (s/select (s/child (s/class :author)) htree)
        first :content first)

    :bookmarks
    (s/select (s/descendant
               (s/id :forum) (s/tag :tbody) (s/tag :tr)) htree)

    :pagecount
    (-> (s/select (s/descendant
                   (s/class :pages) (s/tag :option)) htree)
        last :content first Integer/parseInt)

    :title
    (-> (s/select (s/child (s/tag :title)) htree)
        first :content first
        (string/replace #" - The Something Awful Forums" ""))

    :thread
    (let [thread-tree (first (s/select (s/descendant
                                        (s/id :thread)) htree))
          td-classes [:userinfo :postdate :postbody]]
      (for [class-key td-classes]
        (s/select (s/descendant
                   (s/and (s/tag :td) (s/class class-key))) thread-tree)))))

(defn processed-element [kw elem]
  (case kw
    :bookmark
    (when-let [link (first
                     (s/select (s/descendant
                                (s/class :info) (s/tag :a)) elem))]
      (let [thread-id (re-find #"\d+$" (:href (:attrs link)))
            title (-> link :content first string/trim)]
        {:id thread-id :title title}))

    :postbody
    (hickory-to-hiccup (hickory-div (:content elem) "postbody"))

    :postdate
    (string/trim (last (hickory-to-hiccup elem)))

    :userinfo
    (let [ui (first (s/select (s/descendant (s/tag :dl)) elem))
          author (-> (s/select (s/descendant (s/class :author)) ui)
                     first :content first)
          regdate (-> (s/select (s/descendant (s/class :registered)) ui)
                      first :content first)
          title (-> (s/select (s/descendant (s/class :title)) ui) first)
          avatar (-> (s/select (s/descendant (s/tag :img)) title) first)]
      {:author author
       :regdate regdate
       :avatar-title (hickory-to-hiccup
                      (hickory-div (:content title) "avatar-title"))
       :avatar (when avatar (hickory-to-hiccup avatar))})))

(defn thread-map [id page doc]
  (let [htree       (hickory-doc doc)
        title       (element-node :title htree)
        page-count  (element-node :pagecount htree)
        thread-tree (element-node :thread htree)
        [userinfo postdate postbody] thread-tree]
    {:title title
     :id id
     :page page
     :page-count page-count
     :content
     (for [[ui pd pb] (partition 3 (interleave userinfo postdate postbody))
           :when (not= "Adbot" (element-node :author ui))]
       {:ui (processed-element :userinfo ui)
        :pd (processed-element :postdate pd)
        :pb (processed-element :postbody pb)})}))

(defn bookmarks-map [page doc]
  (let [htree       (hickory-doc doc)
        title       (element-node :title htree)
        page-count  (element-node :pagecount htree)
        bookmarks-tree (element-node :bookmarks htree)]
    {:title title
     :page page
     :page-count page-count
     :content
     (for [row bookmarks-tree
           :let [parsed-row (processed-element :bookmark row)]
           :when parsed-row]
       parsed-row)}))
